/*_############################################################################
  _## 
  _##  SNMP4J-AgentJMX - EnumStringType.java  
  _## 
  _##  Copyright (C) 2006-2009  Frank Fock (SNMP4J.org)
  _##  
  _##  This program is free software; you can redistribute it and/or modify
  _##  it under the terms of the GNU General Public License version 2 as 
  _##  published by the Free Software Foundation.
  _##
  _##  This program is distributed in the hope that it will be useful,
  _##  but WITHOUT ANY WARRANTY; without even the implied warranty of
  _##  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  _##  GNU General Public License for more details.
  _##
  _##  You should have received a copy of the GNU General Public License
  _##  along with this program; if not, write to the Free Software
  _##  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, 
  _##  MA  02110-1301  USA
  _##  
  _##########################################################################*/
package org.snmp4j.agent.mo.jmx.types;

import javax.management.*;

public class EnumStringType extends TypedAttribute {

  private Enum[] enumValues;
  private Class enumClass;
  private int offset = 0;

  public EnumStringType(String name, Class enumClass, Enum[] enumValues) {
    super(name, Integer.class);
    this.enumClass = enumClass;
    this.enumValues = enumValues;
  }

  public Enum[] getEnumValues() {
    return enumValues;
  }

  public int getOffset() {
    return offset;
  }

  public Object transformFromNative(Object nativeValue, ObjectName objectName) {
    Enum en;
    if (nativeValue instanceof Enum) {
      en = (Enum)nativeValue;
    }
    else {
      en = Enum.valueOf(enumClass, nativeValue.toString());
    }
    return offset + en.ordinal();
  }

  public Object transformToNative(Object transformedValue,
                                  Object oldNativeValue, ObjectName objectName) {
    int i = ((Integer)transformedValue).intValue();
    return enumValues[i-offset];
  }

}
